package meerkat.registry;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.VoterRegistry;
import meerkat.util.TimestampComparator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import static meerkat.util.BulletinBoardUtils.GetListOfTags;
import static meerkat.util.BulletinBoardUtils.findTagWithPrefix;

/**
 * TODO : add logging and verification of the messages by Arbel Peled
 */

/**
 * Created by Vladimir Eliezer Tokarev  on 2/19/2016.
 * Gets latest data from given List<BulletinBoardMessage>
 */
public class LatestMessagesCallBack implements FutureCallback<List<BulletinBoardMessage>> {
    public FutureCallback callback;
    protected Collection<DigitalSignature> validators;
    protected String type;

    /**
     * init MessagesCallBack
     * @param callback  voter registry callback object
     * @param validators DigitalSignature object
     */
    public LatestMessagesCallBack(FutureCallback callback,
                                  Collection<DigitalSignature> validators, String type) {
        this.callback = callback;
        this.validators = validators;
        this.type = type;
    }

    /**
     *  Checks if the list of messages is of type GROUP_ID_TAG if it is then calls
     *          handleResult with mapping of the latest groups, else calls to handleResult with
     *          the latest tag from this list (in case of personal data)
     *
     * @param messages List<BulletinBoardAPI.BulletinBoardMessage>
     */
    @Override
    public void onSuccess(List<BulletinBoardMessage> messages) {
        BulletinBoardMessage lastAddedMessage = Collections.max(messages, (first, second) -> {
            TimestampComparator comparator = new TimestampComparator();
            return comparator.compare(first.getMsg().getTimestamp(), second.getMsg().getTimestamp());
        });

        List<BulletinBoardMessage> lastMessageList = new ArrayList<BulletinBoardMessage>(){{add(lastAddedMessage);}};
        switch (type){
            case "getGroups" :
                callback.onSuccess(GetListOfTags(lastMessageList, RegistryTags.GROUP_ID_TAG));
                break;
            case "getVoter" :
                VoterRegistry.VoterInfo info = VoterRegistry.VoterInfo.newBuilder()
                        .setId(VoterRegistry.VoterID.newBuilder()
                                .setId(findTagWithPrefix(lastAddedMessage, RegistryTags.ID_TAG)))
                        .setInfo(lastAddedMessage.getMsg().getData().toStringUtf8()).build();

                callback.onSuccess(info);
                break;
            case "hasVoted" :
                callback.onSuccess(findTagWithPrefix(lastAddedMessage,
                    RegistryTags.VOTE_ACTION_TAG) != null);
                break;
        }
    }

    /**
     * Calls the callback handleResult method with false because the post method failed
     * @param t the exception data that have been thrown during the failure of the post method
     */
    @Override
    public void onFailure(Throwable t) {
        callback.onFailure(t);
    }
}
